@@ -7,7 +7,7 @@
  *      definitions for the pqxx::CachedResult class and support classes.
  *   pqxx::CachedResult is a lazy-fetching, transparently-cached result set
  *
- * Copyright (c) 2001-2002, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2003, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  *-------------------------------------------------------------------------
  */
@@ -73,24 +73,11 @@ public:
     return GetBlock(BlockFor(i)).at(Offset(i));
   }
 
-  /// Number of rows in result set.  
-  /** Figuring out the size of the result set for the first time may take a lot
-   * of time and network traffic, as the CachedResult's internal cursor scans
-   * back and forth in search of the set's last row.  Some 30 blocks of data
-   * may be fetched in the process.
-   */
-  size_type size() const
-  {
-    if (m_Size == -1) DetermineSize();
-    return m_Size;
-  }
-
+  /// Number of rows in result set.  First call may be slow.
+  size_type size() const;
   
   /// Is the result set empty, i.e. does it contain no rows?  May fetch 1 block.
-  bool empty() const
-  {
-    return (m_Size == 0) || (m_Cache.empty() && GetBlock(0).empty());
-  }
+  bool empty() const;
 
   /// Drop all data in internal cache, freeing up memory.
   void clear();
@@ -110,6 +97,8 @@ public:
 
 private:
 
+  typedef Cursor::pos pos;
+
   class CacheEntry
   {
     int m_RefCount;
@@ -124,8 +113,12 @@ private:
   };
 
 
-  blocknum BlockFor(size_type Row) const { return Row / m_Granularity; }
-  size_type Offset(size_type Row) const { return Row % m_Granularity; }
+  blocknum BlockFor(size_type Row) const throw () 
+  	{ return Row / m_Granularity; }
+  size_type Offset(size_type Row) const throw ()
+  	{ return Row % m_Granularity; }
+  Cursor::size_type FirstRowOf(blocknum Block) const throw ()
+    	{ return Block*m_Granularity; }
 
   void MoveTo(blocknum) const;
 
@@ -141,13 +134,6 @@ private:
     return Fetch();
   }
 
-  /** Figure out how big our result set is.  This may take some scanning back
-   * and forth, since there's no direct way to find out.  We keep track of the
-   * highest block known to exist (in m_Lower) and the lowest block known not 
-   * to exist (in m_Upper) to narrow the search range as much as possible.
-   */
-  void DetermineSize() const;
-
   /// Block size.
   size_type m_Granularity;
 
@@ -156,19 +142,6 @@ private:
 
   mutable Cursor m_Cursor;
 
-  /// Current cursor position (in blocks), or -1 if unknown.
-  mutable blocknum m_Pos;
-
-  /// Result set size in rows, or -1 if unknown.
-  mutable size_type m_Size;
-
-  /** Lower and upper bounds to end-of-data: highest block known to exist, and
-   * lowest block known not to exist, respectively.  Once size is known, the
-   * two will be adjacent, with m_Lower referring to the last, probably non-full
-   * block and m_Upper equal to m_Lower+1.
-   */
-  mutable blocknum m_Lower, m_Upper;
-
   // Not allowed:
   CachedResult();
   CachedResult(const CachedResult &);
